from typing import List, Optional
from langchain_plainid import CategoryClassifierProvider
from langchain_core.prompts import ChatPromptTemplate
from langchain_core.output_parsers import StrOutputParser
import logging

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)



CLASSIFICATION_PROMPT = """
    You are a smart agent with an ability to classify text into categories.
    You will be given a text and you will classify it into one of the categories.
    If the text does not match any category, respond with "Other".
    Categories: {categories}
    Text: {query}
    Respond with only the top category name."""

class LLMCategoryClassifierProvider(CategoryClassifierProvider):
    def __init__(self, llm ,prompt_template = None):
        super().__init__()
        self.llm = llm
        if prompt_template is None:
            self.prompt_template = CLASSIFICATION_PROMPT
        else:
            self.prompt_template = prompt_template
        
    def classify(self, query: str, all_categories: List[str], allowed_categories: List[str]) -> Optional[str]:
        all_categories_lower = [c.lower() for c in all_categories]
        input_categories_lower = [c.lower() for c in allowed_categories]
        
        prompt = ChatPromptTemplate.from_template(self.prompt_template)
        chain = prompt | self.llm  | StrOutputParser()
        top_category = chain.invoke({"query": query, "categories": all_categories_lower, "allowed_categories": input_categories_lower})
        logger.info(f"LLM classification result: {top_category}")
        categories_map = {category.lower(): category for category in allowed_categories}
        if top_category.lower() in categories_map:
            matched_category = categories_map[top_category.lower()]
            logging.debug(
                f"Classified '{query}' as '{matched_category}'"
            )
            return query
        raise ValueError(
            f"Top category '{top_category}' not in allowed categories: {allowed_categories}"
        )



import logging
from typing import Any, Optional

from langchain_core.runnables import Runnable, RunnableConfig

from .category_classifier_provider import CategoryClassifierProvider
from .plainid_exceptions import PlainIDCategorizerException
from .plainid_permissions_provider import PlainIDPermissionsProvider


class PlainIDCategorizer(Runnable[str, str]):
    def __init__(
        self,
        classifier_provider: CategoryClassifierProvider,
        permissions_provider: PlainIDPermissionsProvider,
        all_categories: Optional[list[str]] = None,
    ):
        self.classifier_provider = classifier_provider
        self.permissions_provider = permissions_provider
        self.all_categories = all_categories or []

    def invoke(
        self, input: str, config: Optional[RunnableConfig] = None, **kwargs: Any
    ) -> str:
        try:
            allowed = self.permissions_provider.get_allowed_categories()
            category = self.classifier_provider.classify(
                input, self.all_categories, allowed
            )

            if category is None:
                raise PlainIDCategorizerException(
                    "Failed to classify input into a valid category"
                )

            logging.debug(f"Categorizer result: {category}")
            return category
        except PlainIDCategorizerException:
            # Re-raise categorizer exceptions
            raise
        except Exception as e:
            # Wrap other exceptions
            raise PlainIDCategorizerException("Error during categorization", e)

    async def ainvoke(
        self, input: str, config: Optional[RunnableConfig] = None, **kwargs: Any
    ) -> str:
        """Asynchronously process a single string input."""
        import asyncio
        from functools import partial

        # Use run_in_executor to run invoke method in a separate thread
        loop = asyncio.get_event_loop()
        return await loop.run_in_executor(
            None, partial(self.invoke, input, config, **kwargs)
        )
